
module coords

! ********************************************************************* !
!                                                                       !
!                 Defintions of fitting coordinates                     !
!                                                                       !
! ********************************************************************* !


  use trigonom
  implicit none

  double precision, private :: alphaexp = 0.1d0, coshexp = 0.25d0 ! positive

contains

! ==F================================================================F== !
function coscoord(alp) result(f)
  implicit none
  double precision :: alp,f
  intent(in) :: alp

  f = (dcos(alp)+1.0d0)/2.0d0

end function

! ==F================================================================F== !
function coscoord_der(alp) result(f)
  implicit none
  double precision :: alp,f
  intent(in) :: alp

  f = dcos_der(alp)/2.0d0

end function

! ==F================================================================F== !
function coscoord_der_der(alp) result(f)
  implicit none
  double precision :: alp,f
  intent(in) :: alp

  f = dcos_der_der(alp)/2.0d0

end function

! ==F================================================================F== !
function sincoord(alp) result(f)
  implicit none
  double precision :: alp,f
  intent(in) :: alp

  f = dsin(alp)**2.0

end function


! ==F================================================================F== !
function sincoord_der(alp) result(f)
  implicit none
  double precision :: alp,f
  intent(in) :: alp

  f = 2.0d0*dsin(alp)*dsin_der(alp)

end function

! ==F================================================================F== !
function sincoord_der_der(alp) result(f)
  implicit none
  double precision :: alp,f
  intent(in) :: alp

  f = 2.0d0*dsin_der(alp)**2.0+2.0d0*dsin(alp)*dsin_der_der(alp)

end function


! ==F================================================================F== !
function expcoord(r) result(f)
  implicit none
  double precision :: r,f
  intent(in) :: r

  f = 1.0d0 - exp(-alphaexp*r)

end function

! ==F================================================================F== !
function expcoord_der(r) result(f)
  implicit none
  double precision :: r,f
  intent(in) :: r

  f = alphaexp*exp(-alphaexp*r)

end function

! ==F================================================================F== !
function expcoord_der_der(r) result(f)
  implicit none
  double precision :: r,f
  intent(in) :: r

  f = -alphaexp**2.0*exp(-alphaexp*r)

end function

! ==F================================================================F== !
function coshcoord(r) result(f)
  implicit none
  double precision :: r,f
  intent(in) :: r

  f = 1.0d0/cosh(coshexp*r)

end function

! ==F================================================================F== !
function coshcoord_der(r) result(f)

  implicit none
  double precision :: r,f
  intent(in) :: r

  f = -coshexp/(cosh(coshexp*r))**2.0*(sinh(coshexp*r))

end function

! ==F================================================================F== !
function coshcoord_der_der(r) result(f)

  implicit none
  double precision :: r,f
  intent(in) :: r

  ! I would verify this one 
  f = -coshexp**2/(cosh(coshexp*r)**2-2.0*sinh(coshexp*r)**2)/cosh(coshexp*r)**3 

end function

! ==F================================================================F== !
subroutine convert_one(ctype,rold,rnew)

  implicit none
  character (len=1), intent(in) :: ctype
  double precision, intent(in)  :: rold
  double precision, intent(out) :: rnew

  select case (ctype)
    case ("r")
      rnew = expcoord(rold)
    case ("a")
      rnew = coscoord(rold)     
    case ("g")
      rnew = sincoord(rold)
    case ("c")
      rnew = coshcoord(rold)
    case default
       write(*,*) ctype, " transformation is not known. stop " 
       stop
  end select

end subroutine

! ==F================================================================F== !
function convert_der(ctype,rold) result(rder)

  implicit none
  character (len=1), intent(in) :: ctype
  double precision, intent(in)  :: rold
  double precision :: rder

  select case (ctype)
    case ("r")
      rder = expcoord_der(rold)
    case ("a")
      rder = coscoord_der(rold)
    case ("g")
      rder = sincoord_der(rold)
    case ("c")
      rder = coshcoord_der(rold)
    case default
       write(*,*) ctype, " transformation is not known. stop "
       stop
  end select

end function

! ==F================================================================F== !
function convert_der_der(ctype,rold) result(rder)

  implicit none
  character (len=1), intent(in) :: ctype
  double precision, intent(in)  :: rold
  double precision :: rder

  select case (ctype)
    case ("r")
      rder = expcoord_der_der(rold)
    case ("a")
      rder = coscoord_der_der(rold)
    case ("g")
      rder = sincoord_der_der(rold)
    case ("c")
      rder = coshcoord_der_der(rold)
    case default
       write(*,*) ctype, " is wrong in second derivative. stop "
       stop
  end select

end function

end module


