
module curve1d 

! ****************************************************************************** !
!                                                                                !
! 1D Reproducing Kernel Hilbert spaces interpolation method for potential curves !
!                                                                                !
! ****************************************************************************** !


  use kernel
  implicit none
  integer, parameter,private :: max1d = 50, maxwork1d=1000

  type curve
     integer npoints
     double precision, dimension(max1d) :: x,f,c
     type (kernel_def) :: qqq
     character (len=1) :: spec
  end type


  interface read
    module procedure Read1d
  end interface

  interface print
    module procedure Print1d
  end interface

  interface shift
    module procedure shift1d
  end interface

  interface read_stream
    module procedure read_curve
  end interface

  interface print_stream
    module procedure print_curve
  end interface
  
contains

! ==F================================================================F== !
subroutine set_kernel_type(this,spec)
  implicit none
  type (curve), intent(inout) :: this
  character (len=1) :: spec

  this%spec = spec
  call set_kernel(this%qqq,spec)
  
end subroutine


! ==F================================================================F== !
Subroutine Read1d(this,fname)
  implicit none
  type (curve), intent(inout) :: this
  character(len=*), intent(in), optional :: fname
  integer i

  if (present(fname)) then
     open(2,file=fname,action="read")
     read(2,*) this%npoints
     do i=1,this%npoints
       read(2,*) this%x(i), this%f(i)
     end do
     close(2)
  else
     read(*,*) this%npoints
     do i=1,this%npoints
       read(*,*) this%x(i), this%f(i) 
     end do
  end if

end subroutine


! ==F================================================================F== !
Subroutine Print1d(this,fname)
  implicit none
  type (curve), intent(in) :: this
  character(len=*), intent(in), optional :: fname
  integer i

  if (present(fname)) then
     open(2,file=fname,action="write")
     write(2,*) this%npoints
     do i=1,this%npoints
       write(2,*) this%x(i), this%f(i)
     end do
     close(2)
  else
     write(*,*) this%npoints
     do i=1,this%npoints
       write(*,*) this%x(i), this%f(i)
     end do
  end if

end subroutine


! ==F================================================================F== !
Subroutine Shift1d(this,v)
   implicit none
   type (curve), intent(inout) :: this
   double precision, intent(in) :: v 
   
   integer i
   do i=1,this%npoints
    this%f(i)=this%f(i)+v
   end do

end subroutine


! ==F================================================================F== !
function rkhs1d_value(this,x) result (eval)
  implicit none
  type (curve), intent(in) :: this
  double precision :: x, eval
  integer i

  eval=0.0d0
  do i=1,this%npoints
    eval=eval+this%c(i)*kernel_value(this%qqq,this%x(i),x)
  end do

end function

! ==F================================================================F== !
function rkhs1d_value_der(this,x) result (eval)
  implicit none
  type (curve), intent(in) :: this
  double precision :: x, eval
  integer i

  eval=0.0d0
  do i=1,this%npoints
    eval=eval+this%c(i)*kernel_value_der(this%qqq,this%x(i),x)
  end do

end function

! ==F================================================================F== !
Subroutine read_curve(nstream,this)
  implicit none
  type (curve), intent(inout) :: this
  integer, intent(in) :: nstream

  integer i

  read(nstream,*) this%spec
  read(nstream,*) this%npoints
  do i=1,this%npoints
    read(nstream,*) this%x(i), this%f(i), this%c(i)
  end do
  call set_kernel(this%qqq,this%spec)
  

end subroutine

! ==F================================================================F== !
Subroutine print_curve(nstream,this)
  implicit none
  type (curve), intent(in) :: this
  integer, intent(in) :: nstream

  integer i

  write(nstream,*) this%spec
  write(nstream,*) this%npoints
  do i=1,this%npoints
    write(nstream,fmt="(2F25.16,E25.16)") this%x(i), this%f(i), this%c(i)
  end do

end subroutine


! ==F================================================================F== !
Subroutine sort_curve(this,s)
!
! To sort curve data. Sign of s controls ascending or descending order 
!
  implicit none
  type (curve), intent(inout) :: this
  double precision, intent(in) :: s

  double precision, dimension(1:this%npoints) :: weights
  double precision :: xmin, temp
  integer :: i,j, kmin


  do i=1,this%npoints
    weights(i) = s*this%x(i)
  end do

  do i=1,this%npoints-1
    xmin = this%x(i)
    kmin = i
    do j=i+1,this%npoints
      if (this%x(j) < xmin) then
        xmin = this%x(j)
        kmin = j
      end if
    end do
    temp = this%x(i); this%x(i) = this%x(kmin); this%x(kmin) = temp
    temp = this%f(i); this%f(i) = this%f(kmin); this%f(kmin) = temp
  end do

end subroutine

! ==F=================================================================F== !
Subroutine add_points(this,xnew,fnew,s)
!
! Sign of s controls ascending or descending order in which new curve data will be sorted
!
  implicit none
  type (curve), intent(inout) :: this
  double precision, dimension(:), intent(in) :: xnew,fnew
  double precision, intent(in) :: s

  integer :: i

  do i=1,Size(xnew)
    this%x(this%npoints+i) = xnew(i)
    this%f(this%npoints+i) = fnew(i)
  end do

  this%npoints=this%npoints+Size(xnew)

  call sort_curve(this,s)

end subroutine

end module


