 
module data_manip

! ************************************************************************* !
!                                                                           !
!   Some preliminary data manipulations : reading, decomposition, sorting   !
!                                                                           !
! ************************************************************************  !

   use curve1d
   use kernel
   use trigonom
   implicit none
   integer, parameter, private :: maxpoints = 1500

   type fdata
      integer :: npoints
      double precision, dimension(maxpoints) :: r1,r2,r3,v,v3
      character (len=3) :: coordtype
   end type

   interface print
     module procedure print_fdata
   end interface

   interface read
     module procedure read_fdata
   end interface

contains

! ==F================================================================F== !
Subroutine print_fdata(this,filename)
!
! print all but 3-body energies
!
  type (fdata), intent(in) :: this
  character(len=*), intent(in), optional :: filename
  integer i
  if (.not.present(filename)) then
    do i=1,this%npoints
      write(*,fmt="(4F25.16)") this%r1(i), this%r2(i), this%r3(i), this%v(i)
    end do
  else
    open(2,file=filename,action='write')
    do i=1,this%npoints
      write(2,fmt="(4F25.16)") this%r1(i), this%r2(i), this%r3(i), this%v(i)
    end do
    close(2)
  end if
end Subroutine

! ==F================================================================F== !
Subroutine print_all(this,filename)
!
! print all info to file
!
  type (fdata), intent(in) :: this
  character(len=*), intent(in), optional :: filename
  integer i
  if (.not.present(filename)) then
    do i=1,this%npoints
      write(*,fmt="(5F25.16)") this%r1(i), this%r2(i), this%r3(i), this%v(i), this%v3(i)
    end do
  else
    open(2,file=filename,action='write')
    do i=1,this%npoints
      write(2,fmt="(5F25.16)") this%r1(i), this%r2(i), this%r3(i), this%v(i), this%v3(i)
    end do
    close(2)
  end if
end Subroutine

! ==F================================================================F== !
subroutine read_fdata(this,filename)
!
! Read all data from file
!
  implicit none
  type (fdata), intent(out) :: this
  character(len=*), intent(in), optional :: filename
  integer :: nstream, cnt
  logical :: forever
  double precision :: r1,r2,v,r3

  if (present(filename)) then
    open(2,file=filename)
    nstream=2
  else
    nstream=5
  end if

  forever=.true.
  cnt=0
  do while (forever)
      read(nstream,*,end=120,err=120) r1,r2,r3,v
      cnt=cnt+1
      this%r1(cnt) = r1
      this%r2(cnt) = r2
      this%r3(cnt) = r3
      this%v(cnt)  = v
  end do

120   this%npoints=cnt

  if (present(filename)) then
    close(2)
  end if

end subroutine

! ==F================================================================F== !
Subroutine makesymm(this)   
!
! additional symmetrical points for r1,r2
!
   implicit none
   type (fdata), intent(inout) :: this
   
   integer i,cnt
   type (fdata) :: newgr

   newgr = this
   cnt=0
  
   do i=1,this%npoints
        cnt=cnt+1
        newgr%r1(cnt)  = this%r1(i)
        newgr%r2(cnt)  = this%r2(i)
        newgr%r3(cnt)  = this%r3(i)
        newgr%v(cnt)   = this%v(i)
        newgr%v3(cnt)  = this%v3(i)
        if (this%r1(i) /= this%r2(i)) then
          cnt=cnt+1
          newgr%r1(cnt) = this%r2(i)
          newgr%r2(cnt) = this%r1(i)
          newgr%r3(cnt) = this%r3(i)
          newgr%v(cnt)  = this%v(i)
          newgr%v3(cnt) = this%v3(i) 
        end if
   end do

   newgr%npoints=cnt

   this = newgr
  
end Subroutine

! ==F================================================================F== !
Subroutine decompose(this,onebody,twobody1,twobody2,twobody3)
!
! to compute standard 3-body energies
!
  implicit none
  type (fdata), intent(inout)    :: this
  double precision, intent(in)   :: onebody
  type (curve), intent(in)       :: twobody1, twobody2, twobody3

  double precision :: crd1,crd2,crd3,r1,r2,r3,v1,v2,v3

  integer i

  do i=1,this%npoints
    crd1 = this%r1(i)
    crd2 = this%r2(i)
    crd3 = this%r3(i)
    select case (this%coordtype)
      case ("rra")
        call rra2rrr(crd1,crd2,crd3,r1,r2,r3)
      case ("jac")
        call jac2rrr(crd1,crd2,crd3,r1,r2,r3)
      case default
        write (*,*) this%coordtype," - unknown coord type. Stop. "
    end select

    v1 = rkhs1d_value(twobody1,r1)
    v2 = rkhs1d_value(twobody2,r2)
    v3 = rkhs1d_value(twobody3,r3)

    this%v3(i)=this%v(i)-v1-v2-v3-onebody

  end do

End Subroutine

! ==F==============================================================F== !
Subroutine sort_fdata(this,s1,s2,s3)
!
! absolute values of s1,s2,s3 control order  in which r1,r2,and r3 are sorted 
! signs of s1,s2,s3 control their ascending or descending
!
! Typical values for s1,s2,s3 are (+/-) 1.0d7,1.0d3,1.0d-1
!
  implicit none
  type (fdata), intent(inout) :: this
  double precision, intent(in) :: s1,s2,s3

  double precision, dimension(this%npoints) :: weights

  integer i,j,kmin;
  double precision minweight, dumb

  do i=1,this%npoints
    weights(i)= s1*this%r1(i)+s2*this%r2(i)+s3*this%r3(i)
  end do

  do i=1,this%npoints-1
    kmin=i
    minweight=weights(i)
    do j=i+1,this%npoints
      if (weights(j) < minweight) then
        kmin=j
        minweight=weights(j)
      end if
    end do
    dumb = this%r1(i); this%r1(i) = this%r1(kmin); this%r1(kmin) = dumb
    dumb = this%r2(i); this%r2(i) = this%r2(kmin); this%r2(kmin) = dumb
    dumb = this%r3(i); this%r3(i) = this%r3(kmin); this%r3(kmin) = dumb
    dumb = this%v(i);  this%v(i)  = this%v(kmin);  this%v(kmin)  = dumb
    dumb = this%v3(i); this%v3(i) = this%v3(kmin); this%v3(kmin) = dumb
    dumb = weights(i); weights(i) = weights(kmin); weights(kmin) = dumb
  end do

end subroutine

end module 


