


module fast_sum_eval

! ****************************************************************** !
!  Fast summation algorithm  for RKHS                                !
!  3-dim case                                                        !
! ****************************************************************** !

   use kernel
   use coords
   use fast_sum_def

   implicit none

   private :: eval_fast_pattern

contains

! ==F========================================================================F== !
subroutine read_sigmas(this,filename,ns)
!
!  Reading strcuture from binary file
!
  implicit none
  type (fast_sum_data), intent(out) :: this
  character(len=*),intent(in) :: filename
  integer, optional, intent(in) :: ns

  integer :: nstream

  if (present(ns)) then
    nstream = ns
  else
    nstream = 2
  end if

  open(nstream,file=filename,status='old',access='sequential', form='unformatted')
  read(nstream) this
  close(nstream)

end subroutine

! ==F========================================================================F== !
subroutine findZ_lin(xarr,x,z)
!
! to find proper index z by linear search 
!
  implicit none
  double precision, dimension(:), intent(in) :: xarr
  double precision, intent(in)     :: x
  integer, intent(out) :: z

  integer :: i,np
  logical :: found

  np = Size(xarr)
  found = .false.
  i=1
  do while ((.not.found).and.(i <= np-1))
    if ( (xarr(i) <= x) .and. (x < xarr(i+1)) ) then
       found = .true.
       z = i
    end if
    i = i + 1
  end do

  if (.not.found) then
    if (x < xarr(1)) then
        z = 0
    end if
       if (x >= xarr(np)) then
        z = np
    end if

  end if

end subroutine

! ==F==============================================================F== !
subroutine findZ(xarr,x,z)
!
! to find proper index z by bisection 
!
  implicit none
  double precision, dimension(:), intent(in) :: xarr
  double precision, intent(in)     :: x
  integer, intent(out) :: z

  integer :: i,np,kmin,kmax,kk
  logical :: found

  np = Size(xarr)

  if (x < xarr(1)) then
     z = 0
  else if (x >= xarr(np)) then
     z = np
  else 
     kmin = 1
     kmax = np
     do while ((kmax-kmin) > 1)
       kk = (kmax+kmin)/2
       if ( (xarr(kmin) <= x).and.(x < xarr(kk)) ) then
         kmax = kk
       else
         kmin = kk
       end if
      end do
      z = kmin
  end if

end subroutine


! ==F==============================================================F== !
Subroutine eval_fast(this,r1,r2,r3,vint,vder)
! > 
!  main driver for fast sum evaluation 
! <
   implicit none
   type (fast_sum_data), intent(in)  :: this
   double precision, intent(in)  :: r1,r2,r3
   double precision, intent(out) :: vint
   double precision, dimension(3), intent(out), optional :: vder

   integer :: z1,z2,z3
   double precision :: r1new, r2new, r3new

   call convert_one(this%transtype(1:1),r1,r1new)
   call convert_one(this%transtype(2:2),r2,r2new)
   call convert_one(this%transtype(3:3),r3,r3new)

   call findZ(this%r1(1:this%n1),r1new,z1) 
   call findZ(this%r2(1:this%n2),r2new,z2)
   call findZ(this%r3(1:this%n3),r3new,z3)

   call eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,vint,func_arr,func_arr,func_arr)

   if (present(vder)) then

     call eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,vder(1),func_arr_der,func_arr,func_arr)
     call eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,vder(2),func_arr,func_arr_der,func_arr) 
     call eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,vder(3),func_arr,func_arr,func_arr_der)

     vder(1) = vder(1) * convert_der(this%transtype(1:1),r1)
     vder(2) = vder(2) * convert_der(this%transtype(2:2),r2) 
     vder(3) = vder(3) * convert_der(this%transtype(3:3),r3)

   end if

end Subroutine

! ==F==============================================================F== !
Subroutine eval_fast_secder(this,r1,r2,r3,secder)
! >
!  Second derivative of RKHS interpolant along angle coordinate (r3)
!  Needed to remove singularity in rrr -> rra Jacobian if angle == 180.0
! <
   implicit none
   type (fast_sum_data), intent(in)  :: this
   double precision, intent(in)  :: r1,r2,r3
   double precision, intent(out) :: secder
   

   integer :: z1,z2,z3
   double precision :: r1new, r2new, r3new
   double precision :: fder

   call convert_one(this%transtype(1:1),r1,r1new)
   call convert_one(this%transtype(2:2),r2,r2new)
   call convert_one(this%transtype(3:3),r3,r3new)

   call findZ(this%r1(1:this%n1),r1new,z1)
   call findZ(this%r2(1:this%n2),r2new,z2)
   call findZ(this%r3(1:this%n3),r3new,z3)

   call eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,  fder,func_arr,func_arr,func_arr_der)
   call eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,secder,func_arr,func_arr,func_arr_der_der)

   secder = secder*convert_der(this%transtype(3:3),r3)**2.0 + fder*convert_der_der(this%transtype(3:3),r3)

end Subroutine


! ==F========================================================================F== !
Subroutine eval_fast_pattern(this,r1new,r2new,r3new,z1,z2,z3,vint,fnc1,fnc2,fnc3)
!
! Common part in evaluation of potential and derivatives
!
   implicit none
   type (fast_sum_data), intent(in)  :: this
   double precision, intent(in) :: r1new, r2new, r3new
   integer, intent(in) :: z1, z2, z3
   double precision, intent(out) :: vint
   interface
      double precision function fnc1(nn,xx)
         integer, intent(in) :: nn
         double precision, intent(in) :: xx
      end function
      double precision function fnc2(nn,xx)
         integer, intent(in) :: nn
         double precision, intent(in) :: xx
      end function
      double precision function fnc3(nn,xx)
         integer, intent(in) :: nn
         double precision, intent(in) :: xx
      end function
   end interface

   integer :: k,m1,m2,k1,k2
   double precision ::  temp, s1,s2,s3

   double precision, dimension(1:3, 1:mmax, 1:3, 1:mmax, 1:3, 1:mmax) :: ggg
   double precision, dimension(1:3, 1:mmax, 1:3, 1:mmax) :: gg
   double precision, dimension(1:3, 1:mmax) :: g

   ! (1) 6-index gamma's from appropriate sigma set

   ggg(1:3,1:mmax,1:3,1:mmax,1:3,1:mmax) = this%zeta(z1,z2,z3)%sigma(1:3,1:mmax,1:3,1:mmax,1:3,1:mmax)
   
   !
   ! to correct extrapolation at short distances range
   ! s1, s2, s3 control either to include summation over F(3,k) function  
   ! s_i = 0 if corresponding coordinate is less than r_i(1)
   !

   s1 = 1.0d0; s2 = 1.0d0; s3 = 1.0d0

   if (z1 == 0) then
     s1 = 0.0d0
   end if

   if (z2 == 0) then
     s2 = 0.0d0
   end if

   if (z3 == 0) then
     s3 = 0.0d0
   end if

   ! (2) 4-index gamma's 

   do m1=1,3
     do k1=1,mmax
       do m2=1,3
         do k2=1,mmax

           temp = 0.0d0
           do k=1,this%m3(1)
               temp = temp + fnc3(this%fnum3(1,k),r3new)*ggg(m1,k1,m2,k2,1,k) 
           end do
           do k=1,this%m3(2)
               temp = temp + s3*fnc3(this%fnum3(3,k),r3new)*ggg(m1,k1,m2,k2,2,k) &
                           +    fnc3(this%fnum3(2,k),r3new)*ggg(m1,k1,m2,k2,3,k)
           end do
           gg(m1,k1,m2,k2) = temp          

         end do
       end do
     end do
   end do

   ! (3) 2-index gamma's

   do m1=1,3
     do k1=1,mmax
     
       temp = 0.0d0
       do k=1,this%m2(1)
         temp = temp + fnc2(this%fnum2(1,k),r2new)*gg(m1,k1,1,k)
       end do
       do k=1,this%m2(2)
         temp = temp + s2*fnc2(this%fnum2(3,k),r2new)*gg(m1,k1,2,k) &
                     +    fnc2(this%fnum2(2,k),r2new)*gg(m1,k1,3,k)
       end do
       g(m1,k1) = temp

     end do
   end do
   
   ! (4) Potential

   vint = 0.0d0

   do k=1,this%m1(1)
     vint = vint + fnc1(this%fnum1(1,k),r1new)*g(1,k)
   end do

   do k=1,this%m1(2)
     vint = vint + s1*fnc1(this%fnum1(3,k),r1new)*g(2,k) &
                 +    fnc1(this%fnum1(2,k),r1new)*g(3,k)
   end do
   

end subroutine


end module
