
module kernel

! ****************************************************************** !
!                                                                    !
!  Definition of kernel function for fast summation algorithm        !
!  [T.Hollebeek, T-.S. Ho, and H. Rabitz, JCP, 106, 7223 (1997)]     !
!                                                                    !
! ****************************************************************** !


  implicit none
  integer, parameter, private :: mmax = 2

  type kernel_def
     integer, dimension(1:mmax)              :: m
     double precision, dimension(1:3,1:mmax) :: p
     integer, dimension(1:3,1:mmax)          :: fnum
  end type


contains

! ==F================================================================F== !
subroutine set_kernel(this,spec)
!
! to assign values for m1, m2, p's, f_mk(number of function)
!
   implicit none
   type (kernel_def), intent(out) :: this
   character (len=1) :: spec

   select case (spec)

      case ("d") ! distance-like kernel

        this%m(1)      = 2
        this%p(1,1)    = 0.0d0 ! just to fill the space
        this%p(1,2)    = 0.0d0 !
        this%fnum(1,1) = 3
        this%fnum(1,2) = 3


        this%m(2)      = 2
        this%p(2,1)    = 1.0d0/14.0d0
        this%fnum(2,1) = 3
        this%p(3,1)    = this%p(2,1)
        this%fnum(3,1) = 2

        this%p(2,2)    = -1.0d0/18.0d0
        this%fnum(2,2) = 4
        this%p(3,2)    = this%p(2,2)
        this%fnum(3,2) = 1


      case ("a") ! regular angle-like kernel

        this%m(1)      = 2
        this%p(1,1)    = 1.0d0
        this%p(1,2)    = 1.0d0      
        this%fnum(1,1) = 3
        this%fnum(1,2) = 4

        this%m(2) = 2
        this%p(2,1)    = 2.0d0
        this%p(2,2)    = -2.0d0/3.0d0
        this%fnum(2,1) = 5
        this%fnum(2,2) = 6

        this%p(3,1)    = this%p(2,1)
        this%p(3,2)    = this%p(2,2)
        this%fnum(3,1) = 4
        this%fnum(3,2) = 3

      case ("t") ! Taylor kernel (angle-like kernel without first two terms)
    
        this%m(1)      = 2
        this%p(1,1)    = 0.0d0 ! just to fill the space
        this%p(1,2)    = 0.0d0 !
        this%fnum(1,1) = 3
        this%fnum(1,2) = 3

        this%m(2) = 2
        this%p(2,1)    = 2.0d0
        this%p(2,2)    = -2.0d0/3.0d0
        this%fnum(2,1) = 5
        this%fnum(2,2) = 6

        this%p(3,1)    = this%p(2,1)
        this%p(3,2)    = this%p(2,2)
        this%fnum(3,1) = 4
        this%fnum(3,2) = 3

      case default

        write(*,*) " unknown type of kernel function. Stop."
        stop

   end select

end subroutine

! ==F================================================================F== !
function func_arr(nfunc,x) result (f)
!
!  "array" of primitive functions 
!
   implicit none
   double precision :: x, f
   integer :: nfunc
   intent(in) :: nfunc, x

   select case (nfunc)
     case (1)
       f = 1.0d0/x**8
     case (2)
       f = 1.0d0/x**7
     case (3)
       f = 1.0d0
     case (4)
       f = x
     case (5)
       f = x**2
     case (6)
       f = x**3
     case default
       write(*,*) " Wrong index of function in func_arr. Stop."
       stop
   end select

end function 


! ==F================================================================F== !
function func_arr_der(nfunc,x) result (f)
!
!  "array" with derivatives of primitive functions                                             
!
   implicit none
   double precision :: x, f
   integer :: nfunc
   intent(in) :: nfunc, x

   select case (nfunc)
     case (1)
       f = -8.0d0/x**9
     case (2)
       f = -7.0d0/x**8
     case (3)
       f = 0.0d0
     case (4)
       f = 1.0d0
     case (5)
       f = 2.0d0*x
     case (6)
       f = 3.0d0*x**2
     case default
       write(*,*) " Wrong index of function in func_arr_der. Stop."
       stop
   end select

end function

! ==F================================================================F== !
function func_arr_der_der(nfunc,x) result (f)
!
!  "array" with derivatives of primitive functions
!
   implicit none
   double precision :: x, f
   integer :: nfunc
   intent(in) :: nfunc, x

   select case (nfunc)
     case (1)
       f = 72.0d0/x**10
     case (2)
       f = 56.0d0/x**9
     case (3)
       f = 0.0d0
     case (4)
       f = 0.0d0
     case (5)
       f = 2.0d0
     case (6)
       f = 6.0d0*x
     case default
       write(*,*) " Wrong index of function in func_arr_der. Stop."
       stop
   end select

end function

! ==F===============================================================================F== !
function kernel_value(this,x,xprime) result (q)
!
!  Evaluation of 1D kernel for regular summation
!
   implicit none
   type (kernel_def), intent(in) :: this
   double precision :: x, xprime, q
   intent(in) :: x, xprime

   integer  :: k;
   q = 0.0d0
 
   do k=1,this%m(1)
     q = q + this%p(1,k)*func_arr(this%fnum(1,k),x)*func_arr(this%fnum(1,k),xprime)  
   end do

   if ( x <= xprime) then
     do k=1,this%m(2)
       q = q + this%p(2,k)*func_arr(this%fnum(2,k),x)*func_arr(this%fnum(3,k),xprime)
     end do
   else
     do k=1,this%m(2)
       q = q + this%p(2,k)*func_arr(this%fnum(2,k),xprime)*func_arr(this%fnum(3,k),x)
     end do
   end if

   
end function

! ==F================================================================================F== !
function kernel_value_der(this,x,xprime) result (q)
!
!  Evaluation of 1D kernel derivative for regular summation
!
   implicit none
   type (kernel_def), intent(in) :: this
   double precision :: x, xprime, q
   intent(in) :: x, xprime

   integer  :: k;
   q = 0.0d0

   do k=1,this%m(1)
     q = q + this%p(1,k)*func_arr(this%fnum(1,k),x)*func_arr_der(this%fnum(1,k),xprime)
   end do

   if ( x <= xprime) then
     do k=1,this%m(2)
       q = q + this%p(2,k)*func_arr(this%fnum(2,k),x)*func_arr_der(this%fnum(3,k),xprime)
     end do
   else
     do k=1,this%m(2)
       q = q + this%p(2,k)*func_arr_der(this%fnum(2,k),xprime)*func_arr(this%fnum(3,k),x)
     end do
   end if

end function

end module
