
module rkhs_func

! ******************************************************************* ! 
!           Reproducing Kernel Hilbert Spaces method                  !
! ******************************************************************* !

   use kernel
   use data_manip
   use coords
   use trigonom

   implicit none

   integer, parameter, private :: n1max=20, n2max=20, n3max=5, max1d = 20, maxlwork = 100000
     
   type rkdata
      integer :: n1,n2,n3
      double precision, dimension(n1max) :: r1
      double precision, dimension(n2max) :: r2
      double precision, dimension(n3max) :: r3
      double precision, dimension(n1max,n2max,n3max) :: v,alpha  
      character (len=3) :: kerntype,transtype
      type (kernel_def), dimension(3) :: qqq
   end type

   private :: find_uniq

contains

! ==F==============================================================F== !
subroutine find_uniq(inparr,outarr,num)
!
! to find all unique array elements and store them separately 
!
   implicit none
   double precision, dimension(:), intent(in) :: inparr
   double precision, dimension(max1d), intent(out) :: outarr
   integer, intent(out) :: num

   integer :: i,j, cnt
   logical :: found
   double precision, dimension(max1d) :: unique

   cnt = 1
   unique(1) = inparr(1)

   do i=2,Size(inparr)

     found = .false.
     j=1
     do while ((.not.found).and.(j <= cnt))
       if (abs(inparr(i)-unique(j)) < 1.0d-6) then
         found = .true.
       end if
       j=j+1
     end do

     if (.not.found) then
       cnt=cnt+1
       unique(cnt) = inparr(i)
     end if

   end do

   outarr = unique
   num = cnt

end subroutine

! ==F==============================================================F== !
subroutine Extract3body(this,domain)
!
!  transfer 3-body energy from (r1,r2,r3,v,v3) data and store them in matrix form;
!  the original data must be sorted and decomposed in advance  
!
   implicit none
   type (rkdata), intent(out) :: this
   type (fdata), intent(in)   :: domain 

   double precision, dimension(max1d) :: uniq
   integer :: num,i,j,k,cnt

   call find_uniq(domain%r1(1:domain%npoints),uniq,num)
   this%n1 = num
   this%r1(1:num) = uniq(1:num)

   call find_uniq(domain%r2(1:domain%npoints),uniq,num)
   this%n2 = num
   this%r2(1:num) = uniq(1:num)

   call find_uniq(domain%r3(1:domain%npoints),uniq,num)
   this%n3 = num
   this%r3(1:num) = uniq(1:num)

   ! *** Again, we assume the domain to be decomposed and sorted 

   cnt = 0
   do i=1,this%n1
     do j=1,this%n2
       do k=1,this%n3
         cnt = cnt + 1
         this%v(i,j,k)=domain%v3(cnt)
       end do
     end do
   end do

end subroutine

! ==F==============================================================F== !
subroutine ExtractFull(this,domain)
!
! domain must be sorted in advance
!
   implicit none
   type (rkdata), intent(out) :: this
   type (fdata), intent(in)   :: domain

   double precision, dimension(max1d) :: uniq
   integer :: num,i,j,k,cnt

   call find_uniq(domain%r1(1:domain%npoints),uniq,num)
   this%n1 = num
   this%r1(1:num) = uniq(1:num)

   call find_uniq(domain%r2(1:domain%npoints),uniq,num)
   this%n2 = num
   this%r2(1:num) = uniq(1:num)

   call find_uniq(domain%r3(1:domain%npoints),uniq,num)
   this%n3 = num
   this%r3(1:num) = uniq(1:num)

   cnt = 0
   do i=1,this%n1
     do j=1,this%n2
       do k=1,this%n3
         cnt = cnt + 1
         this%v(i,j,k)=domain%v(cnt)
       end do
     end do
   end do

end subroutine


! ==F==============================================================F== !
subroutine print_rkdata(this,filename)
!
!
!
   implicit none
   type (rkdata) :: this
   character (len=*) :: filename
   
   integer :: i,j,k

   open(2,file=filename,action="write")

   do i=1,this%n1
     do j=1,this%n2
       do k=1,this%n3
         write(2,fmt="(3F12.6,2F25.12)") this%r1(i), this%r2(j), this%r3(k), this%v(i,j,k), this%alpha(i,j,k)
       end do
     end do
   end do

   close(2)

end subroutine

! ==F==============================================================F== !
subroutine print_rkdata1(this,filename)
!
!
!
   implicit none
   type (rkdata) :: this
   character (len=*) :: filename
  
   integer :: i,j,k

   open(2,file=filename,action="write")

   do i=1,this%n1
     do j=1,this%n2
       do k=1,this%n3
         write(2,fmt="(3F12.6,F25.12)") this%r1(i), this%r2(j), this%r3(k), this%v(i,j,k)
       end do
     end do
   end do

   close(2)

end subroutine


! ==F==============================================================F== !
subroutine convert_rkdata(this,transtype)
!
! to transform original grid into [0..1]^3 domain 
!
  implicit none
  type (rkdata), intent(inout) :: this
  character (len=3), intent(in) :: transtype

  double precision :: temp
  integer :: i

  this%transtype = transtype

  do i=1,this%n1
     temp = this%r1(i)
     call convert_one(transtype(1:1),temp,this%r1(i))
  end do

  do i=1,this%n2
     temp = this%r2(i)
     call convert_one(transtype(2:2),temp,this%r2(i))
  end do

  do i=1,this%n3
     temp = this%r3(i)
     call convert_one(transtype(3:3),temp,this%r3(i))
  end do

end subroutine

! ==F==============================================================F== !
subroutine find_alpha(this,kerntype,scales)
!
! to find coefficients alpha for regular rkhs summation
!
  implicit none
  type (rkdata), intent(inout) :: this  
  character (len=3) :: kerntype
  double precision, dimension(:), intent(in), optional :: scales

  double precision, dimension(:,:), allocatable :: A,B
  double precision, dimension(:),   allocatable :: S 
  double precision :: rcond=1.0d-20
  integer ia,ja,ka,ib,jb,kb, M,N,NRHS,LDA,LDB,LWORK,rank,info,cnta, cntb
  double precision, dimension(maxlwork) :: WORK
  integer, dimension(:), allocatable :: IPIV
  double precision :: scal
  

  do ia=1,3
    call set_kernel(this%qqq(ia),kerntype(ia:ia))
  end do

  M=this%n1*this%n2*this%n3
  allocate(A(M,M),B(M,1),S(M))
  allocate(IPIV(M))

  cnta = 0
  do ia=1,this%n1 
   do ja=1,this%n2
     do ka=1,this%n3
         cnta = cnta + 1 
         cntb = 0
         do ib=1,this%n1
           do jb=1,this%n2
             do kb=1,this%n3
               cntb = cntb +1
               A(cnta,cntb) = kernel_value(this%qqq(1),this%r1(ia),this%r1(ib)) &
                            * kernel_value(this%qqq(2),this%r2(ja),this%r2(jb)) &
                            * kernel_value(this%qqq(3),this%r3(ka),this%r3(kb)) 
               if (present(scales)) then
                 A(cnta,cntb) = scales(cnta)*A(cnta,cntb)
               end if
             end do
           end do
         end do
         B(cnta,1)=this%v(ia,ja,ka)
     end do
   end do
  end do

  N=M
  LDA=M
  LDB=M
  NRHS=1

  call dgesv(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
  !call dposv('U',N,NRHS,A,LDA,B,LDB,INFO)
  !call dgelss(M,N,NRHS,A,LDA,B,LDB,S,rcond,rank,Work,maxlwork,info)

  if (info /= 0) then
    write(*,*) " ... error in dgelss or dgesv called from find_alpha, info = ", info, ". Stop."
    stop
  end if

  cnta = 0
  do ia=1,this%n1
   do ja=1,this%n2
     do ka=1,this%n3
       cnta = cnta + 1
       this%alpha(ia,ja,ka) = B(cnta,1)
     end do
   end do
  end do

  deallocate(A,B,S)
  deallocate(IPIV)

end subroutine

! ==F================================================================F== !
subroutine average_alpha(this)
!
! make corresponding alpha symmetrical for symmetrical grid data to
! correct descrepancies due to numerical errors  
!
   implicit none
   type (rkdata), intent(inout) :: this

   integer :: i,j,k
   double precision :: av
   logical :: is_symm

   is_symm = (this%n1 == this%n2)
   if (is_symm) then
     do i=1,this%n1
       is_symm = is_symm.and.(abs(this%r1(i)-this%r2(i)) < 1.0d-6)
     end do
   end if

   if (is_symm) then
     do i=1,this%n1
      do j=i,this%n2
       do k=1,this%n3
          av = 0.5d0 * (this%alpha(i,j,k)+this%alpha(j,i,k))
          this%alpha(i,j,k) = av
          this%alpha(j,i,k) = av
       end do
      end do
     end do
   end if


end subroutine


! ==F==============================================================F== !
Subroutine eval_rkhs(this,r1,r2,r3,vint)
!
! to evaluate rkhs function via regular summation 
!
   implicit none
   type (rkdata), intent(in)     :: this
   double precision, intent(in)  :: r1,r2,r3  
   double precision, intent(out) :: vint

   integer :: i,j,k
   double precision :: r1new, r2new, r3new

   call convert_one(this%transtype(1:1),r1,r1new)
   call convert_one(this%transtype(2:2),r2,r2new)
   call convert_one(this%transtype(3:3),r3,r3new)

   vint = 0.0d0
   do i=1,this%n1
     do j=1,this%n2
       do k=1,this%n3
          vint = vint + this%alpha(i,j,k) * kernel_value(this%qqq(1),this%r1(i),r1new) &
                                          * kernel_value(this%qqq(2),this%r2(j),r2new) &
                                          * kernel_value(this%qqq(3),this%r3(k),r3new) 
       end do
     end do
   end do
   
end subroutine

end module
