
module surfitpol
! ===================================================================== !
! Utilities to work with  analytical function from the  SURFIT program  !
! ===================================================================== !

  implicit  none

  type surfpol
     double precision, dimension(286) :: c
     double precision, dimension(3)   :: ro, rp1, rp2, rp3  
     integer, dimension(286,3)        :: index
     integer, dimension(3)            :: icotyp
     integer                          :: npa
  end type

  interface print
    module procedure print_ascii
  end interface

  interface read
    module procedure read_ascii
  end interface

  interface print_stream
    module procedure print_surfpol
  end interface

  interface read_stream
    module procedure read_surfpol
  end interface

contains

! ==F================================================================F== !

subroutine print_ascii(this,p)

    implicit none
    type (surfpol), intent(in) :: this
    character (len=*) , intent(in) :: p

    integer i

    open(2,file=p,action="write")
    write(2,*) this%npa
    write(2,*) this%icotyp(1:3)
    write(2,fmt="(3E20.12)") this%ro
    write(2,fmt="(3E20.12)") this%rp1
    write(2,fmt="(3E20.12)") this%rp2
    write(2,fmt="(3E20.12)") this%rp3
    do i=1,this%npa
      write(2,fmt="(E25.18,3I3)") this%c(i),this%index(i,1:3)
    end do
    close(2)

end subroutine

! ==F================================================================F== !

subroutine read_ascii(this,p)

    implicit none
    type (surfpol), intent(out) :: this
    character (len=*) , intent(in) :: p

    integer i

    open(2,file=p,action="read")
    read(2,*) this%npa
    read(2,*) this%icotyp(1:3)
    read(2,*) this%ro
    read(2,*) this%rp1
    read(2,*) this%rp2
    read(2,*) this%rp3
    do i=1,this%npa
      read(2,*) this%c(i),this%index(i,1:3)
    end do
    close(2)

end subroutine

! ==F================================================================F== !

subroutine load(this,p)

! This one reads binary file from SURFIT

    implicit none
    type (surfpol), intent(out) :: this
    character (len=*) , intent(in) :: p

    double precision, dimension(3,3) :: cpl   
    integer, dimension(3) :: igr,isy,ic
    integer :: molsym


    open(unit=1,file=p,status='OLD',err=88,form='UNFORMATTED')
    read(1,err=88) this%npa,this%c,this%index,this%ro,igr,isy,ic
    goto 5
88  stop 'error loading potential function'

5   this%icotyp(1)=0
    this%icotyp(2)=0
    this%icotyp(3)=0

    read(1,err=3) this%icotyp
    read(1,err=3) this%rp1,this%rp2
    read(1,err=3) this%rp3
    read(1,err=3) molsym,cpl
3   close(1)

end subroutine

! ==F================================================================F== !

! Evaluation of SURFIT functionals and derivatives

function fct(this,rr) result(vv)
   implicit none
   type (surfpol), intent(in) :: this
   double precision, dimension(3) :: rr
   double precision :: vv

   double precision :: f
   double precision, dimension(3) :: qq
   integer i,j

   call qcoord(this,rr,qq)
   vv=0.0d0
   do i=1,this%npa
      f = 1.0d0
      do j=1,3
        if (this%index(i,j) /= 0) then
          f=f*qq(j)**this%index(i,j)
        end if
      end do
      vv=vv+this%c(i)*f
   end do

end function

! ==F================================================================F== !

subroutine qcoord(this,rr,qq)
   implicit none
   type (surfpol), intent(in) :: this 
   double precision, dimension(3) :: rr, qq
   intent(in)  :: rr
   intent(out) :: qq

   integer :: i

   do i=1,3
     qq(i)=rr(i)-this%ro(i)
   end do

   do  i=1,2
     if (this%icotyp(i)==1) then
       qq(i)=qq(i)/rr(i)
     end if
     if (this%icotyp(i)==2) then
        qq(i)=(1.0d0-exp(-this%rp1(i)/this%ro(i)*qq(i)))/this%rp1(i)
     end if
   end do

   if (this%icotyp(3)==3) then
      qq(3)=this%rp1(3)*qq(3)+this%rp2(3)*qq(3)**2+this%rp3(3)*qq(3)**3
   end if

end subroutine

! ==F================================================================F== !
!   !!!!!  !!!!  !!!!  !!!  !!!  works with simple shift coordinate only !! !!! !!
Subroutine fct_der(this,rr,gg) 
   implicit none
   type (surfpol), intent(in) :: this
   double precision, dimension(3) :: rr
   double precision, dimension(3) :: gg

   double precision, dimension(3) :: qq,dd,ff
   double precision, dimension(3) :: zero = (/0.0d0,0.0d0,0.0d0/)
   integer i,j,p

   call qcoord(this,rr,qq)

   gg = zero
   
   do i=1,this%npa
      do j=1,3

        p = this%index(i,j)
        ff(j)=qq(j)**p
        if (p == 0) then
           dd(j)=0.0d0
        else if (p == 1) then
           dd(j)=1.0d0
        else
           dd(j)=p*qq(j)**(p-1)
        end if

      end do

      gg(1)=gg(1)+this%c(i)*dd(1)*ff(2)*ff(3)
      gg(2)=gg(2)+this%c(i)*ff(1)*dd(2)*ff(3)
      gg(3)=gg(3)+this%c(i)*ff(1)*ff(2)*dd(3)

   end do

end subroutine

! ==F================================================================F== !
! This
!   !!!!!  !!!!  !!!!  !!!  !!!  works with simple shift coordinate only !! !!! !!
Subroutine fct_secder(this,rr,secder)
   implicit none
   type (surfpol), intent(in) :: this
   double precision, dimension(3) :: rr
   double precision, intent(out) :: secder

   double precision, dimension(3) :: qq
   double precision :: dd,f1,f2
   integer i,j,p

   call qcoord(this,rr,qq)

   secder = 0.0d0

   do i=1,this%npa
      p  = this%index(i,3)
      f1 = qq(1)**this%index(i,1)
      f2 = qq(2)**this%index(i,2) 
      if (p == 0) then
         dd = 0.0d0
      else if (p == 1) then
         dd = 0.0d0
      else  
         dd = p*(p-1)*qq(3)**(p-2)
      end if
      secder = secder+this%c(i)*f1*f2*dd
   end do

end subroutine


!   ------------------------- streaming  -----------------------------   !

! ==F================================================================F== !

subroutine print_surfpol(nstream,this)

    implicit none
    type (surfpol), intent(in) :: this
    integer, intent(in) :: nstream

    integer i

    write(nstream,*) this%npa
    write(nstream,*) this%icotyp(1:3)
    write(nstream,fmt="(3E20.12)") this%ro
    write(nstream,fmt="(3E20.12)") this%rp1
    write(nstream,fmt="(3E20.12)") this%rp2
    write(nstream,fmt="(3E20.12)") this%rp3
    do i=1,this%npa
      write(nstream,fmt="(E25.18,3I3)") this%c(i),this%index(i,1:3)
    end do

end subroutine

! ==F================================================================F== !

subroutine read_surfpol(nstream,this)

    implicit none
    type (surfpol), intent(out) :: this
    integer, intent(in) :: nstream

    integer i

    read(nstream,*) this%npa
    read(nstream,*) this%icotyp(1:3)
    read(nstream,*) this%ro
    read(nstream,*) this%rp1
    read(nstream,*) this%rp2
    read(nstream,*) this%rp3
    do i=1,this%npa
      read(nstream,*) this%c(i),this%index(i,1:3)
    end do

end subroutine

end module


