! ====================================================== !
!          Procedures to work with matrix                !         
! ====================================================== !

module TMatrix

  implicit none   

contains

! ========================================================= 
subroutine read_matrix(M)
   implicit none
   double precision, intent(out),dimension(:,:) :: M
   integer i,j
   do i=1,size(M,1)
     read(*,*) (M(i,j),j=1,size(M,2))
   end do
end subroutine read_matrix
   
! ====================================================== !
!       to print a matrix in table form                  !
! ====================================================== !
subroutine print_matrix(M,name)
  implicit none
  double precision, dimension(:,:), intent(in) :: M
  character(25), intent(in), optional :: name
  integer i,j

  if (present(name)) then
     write(*,*) name
  end if

  do i=1,size(M,1)
    do j=1,size(M,2)-1
      write(*,fmt="((A2)(F12.6))",advance="no") " ",M(i,j)
    end do
    write(*,fmt="(F12.6)") M(i,size(M,2))
  end do
end subroutine print_matrix


! ====================================================== !
!      interface to Lapack's DGESV procedure             !
! ====================================================== !
subroutine SolveLinEq(A,B,X,info)
  implicit none
  ! adopt all dimension size as it came in

  double precision, dimension(:,:), intent(in) :: A
  double precision, dimension(:,:), intent(in) :: B
  double precision, dimension(:,:), intent(out) :: X
  integer, intent(out) :: info

  ! A - matrix, B - right hand vector
  ! X - solution vector to be found
  ! info - info from DGESV

  double precision, dimension(size(A,1),size(A,2)) :: Acopy
  double precision, dimension(size(B,1),size(B,2)) :: Bcopy
  integer, dimension(size(A,2)) :: IPIV
  integer i,j,N,NRHS,LDA,LDB;

  N = size(A,2)
  NRHS = size(B,2)
  LDA = size(A,1)
  LDB = size(B,1)

  Bcopy=B
  Acopy=A

!  call DGESV(N,NRHS,Acopy,LDA,IPIV,Bcopy,LDB,INFO)
  call dgesv(N,NRHS,Acopy,LDA,IPIV,Bcopy,LDB,INFO)

  X=Bcopy

!  check
!   if (maxval(abs(matmul(A,B)-X))>1.0d-7) then
!      info=7
!   end if
  
end subroutine SolveLinEq

! ========================================================= !
!        interface to Lapack's DGETRF and DGETRI            !
! ========================================================= !

subroutine InverseMatrix(A,info)
  implicit none
  double precision, dimension(:,:), intent(inout) :: A
  integer, intent(out) :: info

  double precision, dimension(size(A,2)) :: IPIV
  double precision, dimension(size(A,1)) :: WORK

  integer M,N,LDA,LWORK

  M = size(A,1)
  N = size(A,2)
  LDA = size(A,1)
  LWORK=N

  call dgetrf(M,N,A,LDA,IPIV,INFO)
  if (INFO == 0) then
    call dgetri(N,A,LDA,IPIV,WORK,LWORK,INFO)
  end if

end subroutine InverseMatrix

! ====================================================== !
!      interface to Lapack DGELSS procedure              !
! ====================================================== !
subroutine SingValDecomp(A,B,X,info)
  implicit none

  ! adopt all dimension size as it came in

  double precision, dimension(:,:), intent(in) :: A
  double precision, dimension(:,:), intent(in) :: B
  double precision, dimension(:,:), intent(out) :: X
  integer, intent(out) :: info

  ! A - matrix, B - right hand vector
  ! X - soluti

  double precision, dimension(size(A,1),size(A,2)) :: Acopy
  double precision, dimension(size(B,1),size(B,2)) :: Bcopy
  double precision, dimension(size(A,1)) :: S
  double precision, allocatable, dimension(:) :: WORK
  integer M,N,NRHS,LDA,LDB,RANK,LWORK
  double precision RCOND

  N = size(A,2)
  m = size(A,1)
  NRHS = size(B,2)
  LDA = size(A,1)
  LDB = size(B,1)

  Bcopy=B
  Acopy=A

  LWORK=10*N
  allocate(WORK(LWORK))

  RCOND=1.0D-14
  call dgelss(M,N,NRHS,Acopy,LDA,Bcopy,LDB,S,RCOND,RANK,WORK,LWORK,INFO)

  X=Bcopy

  deallocate(Work)

end subroutine 

! ====================================================== !
!      interface to Lapack DGELSD procedure              !
! ====================================================== !
subroutine GenLeastSquares(A,B,X,info)
  implicit none

  ! adopt all dimension size as it came in

  double precision, dimension(:,:), intent(in) :: A
  double precision, dimension(:,:), intent(in) :: B
  double precision, dimension(:,:), intent(out) :: X
  integer, intent(out) :: info

  ! A - matrix, B - right hand vector
  ! X - solution

  double precision, dimension(size(A,1),size(A,2)) :: Acopy
  double precision, dimension(size(B,1),size(B,2)) :: Bcopy
  double precision, dimension(size(A,1)) :: S
  double precision, allocatable, dimension(:) :: WORK
  integer, allocatable, dimension(:) :: IWORK
  integer M,N,NRHS,LDA,LDB,RANK,LWORK
  double precision RCOND
  N = size(A,2)
  M = size(A,1)
  NRHS = size(B,2)
  LDA = size(A,1)
  LDB = size(B,1)
  Bcopy=B
  Acopy=A
  LWORK=100*M
  allocate(WORK(LWORK),IWORK(LWORK))
  RCOND=1.0D-14
  call dgelss(M,N,NRHS,Acopy,LDA,Bcopy,LDB,S,RCOND,RANK,WORK,LWORK,IWORK,INFO)
  X=Bcopy
  deallocate(Work,IWORK)
end subroutine 

! ================================================================ !
!                           dllsq                                  !
! ================================================================ !
! Description:
! To solve linear least squares problems, i.e.
! to minimize the euclidean norm of b-a*x.
! This is not full substitute for the DLLSQ
! procedure from IBM SSP library in Fortran,
! but rather strongly simplified variant.
!
! Parameters:
! Ainp - double precision M by N coefficient matrix (in) 
! Binp - double precision M by LL right hand side matrix (in)
! X    - double precision N by LL solution matrix (out)
! info - integer (out) :
!    -2, if Ainp iz zero matrix
!     k, if k=rank(Ainp) < N
!     0, if rank(A)=N
!
! Method:
! Householder transformations are used to transform matrix a
! to upper triangular form. After having applied the same
! transformation to the right hand side matrix b,
! an approximate solution of the problem is computed by back
! substitution.
!
! Ref:
! G. Golub,
! Numerical Method for solving linear least squares problems,
! Numerische Mathematik, 1965, Vol.7, Iss.3, pp.206-216.
!
! ================================================================= !
Subroutine DLLSQ(Ainp,Binp,X,info,eps_inp)
  implicit none
  double precision, dimension(:,:) :: Ainp,Binp,X
  intent(in) :: Ainp,Binp
  intent(out) :: X
  integer, intent(out) :: info
  double precision, intent(in), optional :: eps_inp

  integer i,j,k,l,kpiv, M, N, LL
  double precision beta,sigma,h,eps
  double precision, dimension(Size(Ainp,1),Size(Ainp,2)) :: A
  double precision, dimension(Size(Binp,1),Size(Binp,2)) :: B
  
  double precision, dimension(Size(A,1)) :: u
  double precision, dimension(Size(A,1),Size(A,2)) :: dd
  double precision, dimension(Size(A,1),Size(B,2)) :: cc

  M=Size(A,1);N=Size(A,2);LL=Size(B,2);
  A=Ainp;B=Binp;
  if (present(eps_inp)) then
    eps=eps_inp
  else
    eps=1.0d-11
  end if
  sigma=0.0d0
  info=0
  do i=1,N
    h=0.0d0
    do j=1,N
      h=h+A(j,i)**2.0
    end do
    if (h>sigma) then
      sigma=h
      kpiv=i
    end if
  end do
  if (sigma<=eps) then
     info=-1;return;
  end if

  ! Householder transformations

  do k=1,N
    sigma=0.0d0
    do j=k,M
      sigma=sigma+A(j,k)**2.0
    end do
    sigma=sigma**0.5
    !write(*,*) "sigma=", sigma
    if (sigma<=eps) then
       info=k-1;return;
    end if
    beta=1.0d0/(sigma*(sigma+abs(a(k,k))))
    do i=1,k-1
      u(i)=0.0d0
    end do  
    u(k)=(sigma+abs(a(k,k)))*sign(a(k,k))
    do i=k+1,M
      u(i)=a(i,k)
    end do
    
    ! transform matrix a

    do i=1,M
      do j=1,N
        dd(i,j)=0.0d0
        do l=1,M
           dd(i,j)=dd(i,j)+beta*u(l)*a(l,j)
        end do
        dd(i,j)=dd(i,j)*u(i)
      end do
    end do
    do i=1,M
      do j=1,N
        a(i,j)=a(i,j)-dd(i,j)
      end do
    end do

    ! transform matrix b
    
    do i=1,M
      do j=1,LL
        cc(i,j)=0.0d0
        do l=1,M
          cc(i,j)=cc(i,j)+beta*u(l)*b(l,j)
        end do
        cc(i,j)=cc(i,j)*u(i)
      end do
    end do
    do i=1,M
      do j=1,LL
        b(i,j)=b(i,j)-cc(i,j)
      end do
    end do
  end do

  ! Back Substitution
  
  do k=1,LL
    X(N,k)=b(N,k)/a(N,N)
    do i=N-1,1,-1 
      X(i,k)=b(i,k);
      do j=i+1,N
        X(i,k)=X(i,k)-a(i,j)*x(j,k)
      end do
      x(i,k)=x(i,k)/a(i,i)
    end do
  end do

contains

double precision function Sign(x)
  implicit none
  double precision x
  if (x <0) then
    Sign=-1.0
  else if (x > 0) then
    Sign=1.0
  else
    Sign=0.0
  end if
end function

end Subroutine

! ================================================================= !
integer function rank_matrix(A)
   implicit none
   double precision, dimension(:,:), intent(in) :: A
  
   integer info,i
   double precision, dimension(Size(A,1),1) :: B
   double precision, dimension(Size(A,2),1) :: X
   do i=1,Size(A,1)
     B(i,1)=0.0d0
   end do
   call dllsq(A,B,X,info)
   if (info==0) then
      rank_matrix=Size(A,2)
   else
      rank_matrix=info
   end if
end function

! ========================================================= !
!                    DIAGANALIZATION                        !
! ========================================================= !

subroutine DiagonalizeMatrix(A,D,O,info)
   implicit none
   double precision, intent(in),  dimension(:,:)  :: A ! matrix to diagonalize
   double precision, intent(out), dimension(:)    :: D ! Eigenvalues
   double precision, intent(out), dimension(:,:)  :: O ! Eigenvectors, orthonormalized
   integer info

   integer N,LDA,LWORK
   double precision, allocatable, dimension(:,:) :: AA
   double precision, allocatable , dimension(:) :: WW

   N=size(A,1)
   LDA=N
   LWORK=3*N-1
   allocate(AA(N,N),WW(LWORK))
   AA=A

   call DSYEV('V','L',N,AA,LDA,D,WW,LWORK,info)
   if (info == 0) then
     O=AA
   end if

end subroutine


end module Tmatrix
