
module trigonom
! ************************************************************** !
! some trigonometric functions and transformation of coordinates !
! ************************************************************** !

 implicit none

! ========================================================= !
!                                                           !
!                          B                                !
!                         /|\                               !
!                        / | \                              !
!                       /  |  \                             !
!                      /   |   \                            !
!                     /    |    \                           !
!                    /     |     \                          !
!                   /      |      \                         !
!                  A-------X-------C                        !
!                                                           !
!                                                           !
!      r1=AB, r2=BC, r3=AC, alp=ABC, med=BX, gam=BXA        !
!                                                           !
! ========================================================= !

  private :: pi

contains 

!  ==F=====================================================

double precision function pi()
    implicit none
    pi = acos(-1.0d0)
end function

!  ==F=====================================================

function mySqrt(x) result(f)
   implicit none
   double precision :: x,f
   intent(in) :: x

   f = Sqrt(max(0.0d0,x))

end function

!  ==F===================================================== 

function dcos(theta) result (f) 
  implicit none
  double precision :: theta, f
  f=cos(pi()*theta/180.0d0)
end function

!  ==F=====================================================

function dcos_der(theta) result (f)
  implicit none
  double precision :: theta, f
  f=-sin(pi()*theta/180.0d0)*(pi()/180.0d0)
end function

!  ==F=====================================================

function dcos_der_der(theta) result (f)
  implicit none
  double precision :: theta, f
  f=-cos(pi()*theta/180.0d0)*(pi()/180.0d0)**2.0
end function



!  ==F=====================================================

function dsin(theta) result (f)
  implicit none
  double precision :: theta, f
  f=sin(pi()*theta/180.0d0)
end function

!  ==F=====================================================

function dsin_der(theta) result (f)
  implicit none
  double precision :: theta, f
  f=cos(pi()*theta/180.0d0)*(pi()/180.0d0)
end function

!  ==F=====================================================

function dsin_der_der(theta) result (f)
  implicit none
  double precision :: theta, f
  f=-sin(pi()*theta/180.0d0)*(pi()/180.0d0)**2.0
end function



!  ==F=====================================================

function dacos(f) result (theta)
  implicit none
  double precision f,theta,temp
  temp=max(-1.0d0,min(f,1.0d0))  
  theta=(180.0d0/pi())*acos(temp) 
end function

!  ==F=====================================================

function dasin(f) result (theta)
  implicit none
  double precision f,theta,temp
  temp=max(-1.0d0,min(f,1.0d0))
  theta=(180.0d0/pi())*asin(temp)
end function

!  ==F=====================================================

Subroutine jac2rrr(med,base,gam,r1,r2,r3)

   implicit none
   double precision :: med,base,gam,r1,r2,r3
   intent(in)   :: med,base,gam
   intent(out)  :: r1,r2,r3

   r1 = mySqrt((base/2.0d0)**2.0+med**2.0-base*med*dcos(gam))
   r2 = mySqrt((base/2.0d0)**2.0+med**2.0+base*med*dcos(gam))
   r3 = base

end Subroutine

!  ==F=====================================================

Subroutine rrr2jac(r1,r2,r3,med,base,gam)
   implicit none
   double precision :: base,med,gam,r1,r2,r3
   intent(out)   :: med,base,gam
   intent(in)    :: r1,r2,r3

   double precision :: beta

   base  = r3
   beta  = dacos((r1**2.0+r3**2.0-r2**2.0)/(2*r1*r3))
   med   = mySqrt(r1**2.0+(r3/2.0)**2.0-r1*r3*dcos(beta))
   gam   = dacos(((r3/2.0)**2.0+med**2.0-r1**2.0)/(r3*med))
  
end Subroutine

!==F========================================================

Subroutine rra2rrr(r1,r2,alp,d1,d2,d3)
  implicit none
  double precision :: r1,r2,alp,d1,d2,d3
  intent(in) :: r1,r2,alp
  intent(out) :: d1,d2,d3

  d1=r1
  d2=r2
  d3=mySqrt(r1**2.0+r2**2.0-2.0*r1*r2*dcos(alp))

end Subroutine

!==F========================================================

Subroutine rrr2rra(d1,d2,d3,r1,r2,alp)
  implicit none
  double precision :: d1,d2,d3,r1,r2,alp
  intent(in) :: d1,d2,d3
  intent(out) :: r1,r2,alp

  r1=d1
  r2=d2
  alp=dacos((d1**2.0+d2**2.0-d3**2.0)/(2.0*d1*d2))

end Subroutine


!==F========================================================

Subroutine Cart2rrr(aa,bb,cc,rr,jcb)
!
!
!
  implicit none
  double precision, dimension(3):: aa,bb,cc,rr
  intent(in) :: aa,bb,cc
  intent(out) :: rr
  double precision, dimension(9,3), intent(out), optional :: jcb
  rr(1)=mySqrt((aa(1)-bb(1))**2.0+(aa(2)-bb(2))**2.0+(aa(3)-bb(3))**2.0)
  rr(2)=mySqrt((cc(1)-bb(1))**2.0+(cc(2)-bb(2))**2.0+(cc(3)-bb(3))**2.0)
  rr(3)=mySqrt((cc(1)-aa(1))**2.0+(cc(2)-aa(2))**2.0+(cc(3)-aa(3))**2.0)
  if (present(jcb)) then
    jcb(1,1) =  (aa(1)-bb(1))/rr(1)
    jcb(2,1) =  (aa(2)-bb(2))/rr(1)
    jcb(3,1) =  (aa(3)-bb(3))/rr(1)
    jcb(4,1) = -(aa(1)-bb(1))/rr(1)
    jcb(5,1) = -(aa(2)-bb(2))/rr(1)
    jcb(6,1) = -(aa(3)-bb(3))/rr(1)
    jcb(7,1) =  0.0d0
    jcb(8,1) =  0.0d0
    jcb(9,1) =  0.0d0
    jcb(1,2) =  0.0d0
    jcb(2,2) =  0.0d0
    jcb(3,2) =  0.0d0
    jcb(4,2) = -(cc(1)-bb(1))/rr(2)
    jcb(5,2) = -(cc(2)-bb(2))/rr(2)
    jcb(6,2) = -(cc(3)-bb(3))/rr(2)
    jcb(7,2) =  (cc(1)-bb(1))/rr(2)
    jcb(8,2) =  (cc(2)-bb(2))/rr(2)
    jcb(9,2) =  (cc(3)-bb(3))/rr(2)
    jcb(1,3) = -(cc(1)-aa(1))/rr(3)
    jcb(2,3) = -(cc(2)-aa(2))/rr(3)
    jcb(3,3) = -(cc(3)-aa(3))/rr(3)
    jcb(4,3) =  0.0d0
    jcb(5,3) =  0.0d0
    jcb(6,3) =  0.0d0
    jcb(7,3) =  (cc(1)-aa(1))/rr(3)
    jcb(8,3) =  (cc(2)-aa(2))/rr(3)
    jcb(9,3) =  (cc(3)-aa(3))/rr(3)
  end if
end Subroutine

!==F========================================================
subroutine triangle_converter(spec,rold,rnew,jcb)
!
!
!  
   implicit none
   character (len = *), intent(in)  :: spec
   double precision, dimension(3)   :: rold,rnew
   double precision, dimension(3,3), intent(out), optional :: jcb
   intent(in)  :: rold
   intent(out) :: rnew

   select case(spec)
      case ("rrr->rra")
        call rrr2rra(rold(1),rold(2),rold(3),rnew(1),rnew(2),rnew(3))
      case ("rrr->rrb")
        call rrr2rra(rold(1),rold(3),rold(2),rnew(1),rnew(2),rnew(3))
      case ("rrr->jac")
        call rrr2jac(rold(1),rold(2),rold(3),rnew(1),rnew(2),rnew(3))
      case ("rra->rrr")
        call rra2rrr(rold(1),rold(2),rold(3),rnew(1),rnew(2),rnew(3))
      case ("rrb->rrr")
        call rra2rrr(rold(1),rold(2),rold(3),rnew(1),rnew(3),rnew(2))
      case ("jac->rrr")
        call jac2rrr(rold(1),rold(2),rold(3),rnew(1),rnew(2),rnew(3))
   end select
   
end subroutine

end module trigonom


