
!* * * * * * * * 
!* Generates data for relax triangle coordinate plot 
!* using hyperspherical coordinates
!* [ A.J.C. Varandas, CPL, 138(5), 455 (1987)]
!*


module relax_triangle

   use hgbr2_surf_def
   use hgbr2_surf_main
   use curve1d

   implicit none

   double precision, private :: beta_star,gamma_star

contains

! ==F====================================================F== !
subroutine golden_rule_optim(userfunc,xx,xmin,xmax,eps,ncalls)

   implicit none

   double precision, intent(out) :: xx
   interface
     double precision function userfunc(x)
        double precision :: x
     end function
   end interface
   double precision, intent(in) :: xmin, xmax, eps
   integer, intent(out), optional :: ncalls

   double precision :: ratio,a,b,a1,b1,fa,fb

   if (present(ncalls)) then
     ncalls = 0
   end if

   a=xmin
   b=xmax

   ratio=(sqrt(5.0)-1.0)/2.0
   a1=ratio*a+(1.0-ratio)*b
   b1=ratio*b+(1.0-ratio)*a

   fa=userfunc(a1)
   fb=userfunc(b1)

   if (present(ncalls)) then
     ncalls = ncalls+2
   end if


   do while ((b-a) > eps)
      if ((fa-fb) <= 0.0) then
         b=b1
         b1=a1
         fb=fa
         a1=ratio*a+(1.0-ratio)*b
         fa=userfunc(a1)
      else
         a=a1
         a1=b1
         fa=fb
         b1=ratio*b+(1.0-ratio)*a
         fb=userfunc(b1)
      end if
      if (present(ncalls)) then
        ncalls = ncalls+1
      end if
   end do

   xx=(a+b)/2.0

end subroutine


! ==F====================================================F== !

Function vmin(qq) result(f)
   implicit none
   double precision :: qq,f

   double precision :: s1,s2,s3,r1,r2,r3
   double precision, dimension(3) :: vder


   s1 = beta_star/(2.0*Sqrt(3.0))+1.0/3.0-gamma_star/6.0
   s2 = -beta_star/(2.0*Sqrt(3.0))+1.0/3.0-gamma_star/6.0
   s3 = (gamma_star+1)/3.0

   
   r1 = Sqrt(3.0*qq**2.0*s1)
   r2 = Sqrt(3.0*qq**2.0*s2)
   r3 = Sqrt(3.0*qq**2.0*s3)

   call GlobalPot(r1,r2,r3,f,vder)


end function

! ==F====================================================F== !

Function PlotFunc(beta,gamma,dumb) result (f)

  implicit none
  double precision :: beta,gamma,dumb,f
  intent(in) :: beta,gamma,dumb
 
  double precision :: qopt,eps, qmin, qmax

  
  beta_star  = beta
  gamma_star = gamma

  eps  =  1.0d-7
  qmin =  0.5d0
  qmax = 14.0d0

  if ( (beta_star**2.0 + gamma_star**2.0) <= 1.d0) then
    call golden_rule_optim(vmin,qopt,qmin,qmax,eps)
    f = vmin(qopt)
  else 
    f = 0.0d0
  end if

end function

end module


program main 

  use hgbr2_surf_def
  use relax_triangle

  integer, parameter :: nr = 250, na=360
  double precision :: r,rmin,rmax,rstep,a,amin,amax,astep,gamma,beta,dumb,v,pi
  integer :: i,j

  call read_in(GS,"hgbr2-pot.bin")

  pi = acos(-1.0d0)

  rmin  = 0.0d0
  rmax  = 0.99d0
  rstep = (rmax-rmin)/(nr-1)

  amin  =  0.0d0
  amax  =  2.0d0*pi
  astep = (amax-amin)/(na-1)


  do i=1,na
    do j=1,nr-1
      a     = amin+astep*(i-1)
      r     = rmin+rstep*(j-1)    
      beta  = r*cos(a)
      gamma = r*sin(a)
      v     = 621.51d0 * PlotFunc(beta,gamma,dumb)
      write(*,fmt="(3F15.8)") a,v,r
    end do
    write(*,*)
  end do
  

end program main





