
! ********************************************************!
!  This program collect data from ascii files,
!  makes decompositions, RKHS fits, precompute 
!  info for fast summation algorithm
!  and puts all these in binary file 
! ********************************************************!

module surf_construct

  use trigonom
  use kernel
  use coords
  use switches
  use curve1d
  use curve1d_fit
  use data_manip
  use rkhs_func
  use fast_sum_def
  use fast_sum_init
  use hgbr2_surf_def
  use timing
  use surfitpol

  implicit none 

  type (fdata)    :: mydata
  type (rkdata)   :: myfunc

contains

! == F ===========================================================================
Subroutine set_default_param
  implicit none

  GS%r1eq      =   4.54284d0 ! > Parameters for switch to near-equilibrium patch
  GS%r2eq      =   4.54284d0 !
  GS%alp_eq    = 180.0d0     !
  GS%dlt1eq    =   0.3d0     !
  GS%dlt2eq    =   0.3d0     !
  GS%dlta_eq   =  15.0d0     !
  GS%gamma_eq  =   4.0d0     ! 
  GS%ro_eq     =   1.0d0     !
  GS%singcorr  =   0.001d0   !

  GS%beta_sw   = 115.0d0     ! > Parameters for switch  between different RKHS pieces
  GS%alp_sw    = 115.0d0     !
  GS%beta_g    =   0.3d0     !
  GS%alp_g     =   0.3d0     !

  GS%r1max =  25.0d0
  GS%r2max =  25.0d0
  GS%r3max =  25.0d0

  GS%g1max =  0.5d0
  GS%g2max =  0.5d0
  GS%g3max =  0.5d0


end subroutine


! ==F====================================================F== ! 
Subroutine init_twobody
  implicit none

  ! ---- onebody + two body terms
  open(2,file="hgbr2-oneb.dat",action="read")
  read(2,*) GS%onebody
  close(2)
  
  call read(GS%hgbr,"hgbr.diat")
  call shift(GS%hgbr,-GS%onebody)
  call set_kernel_type(GS%hgbr,"d")
  call fit(GS%hgbr)
  
  call read(GS%brbr,"brbr.diat")
  call shift(GS%brbr,-GS%onebody)
  call set_kernel_type(GS%brbr,"d")
  call fit(GS%brbr)

end subroutine


! ==F====================================================F== ! 
Subroutine init_recomb
  implicit none
  double precision :: d1,d2,d3, en1,en2,en3
  character (len = 30) :: sdate, stime1, stime2
  integer :: i,k

  call read(mydata,"hgbr2-rec.dat")
  mydata%coordtype = "rra"
  call sort_fdata(mydata,1.0d7,1.0d3,-1.0d-1)

  call decompose(mydata,GS%onebody,GS%hgbr,GS%hgbr,GS%brbr)

  call Extract3body(myfunc,mydata)
  call convert_rkdata(myfunc,"rra")
  call find_alpha(myfunc,"dda")
  call average_alpha(myfunc)

  call date_and_time(sdate,stime1)
  call find_sigmas(GS%rec,myfunc)
  call date_and_time(sdate,stime2)

  write(*,*) " Sigmas for recombination piece : ", get_sec(stime2)-get_sec(stime1)," sec."

end subroutine

! ==F====================================================F== ! 
Subroutine init_abstr
  implicit none
  character (len = 30) :: sdate, stime1, stime2

  call read(mydata,"hgbr2-abs.dat")
  mydata%coordtype = "rra"
  call sort_fdata(mydata,1.0d7,1.0d3,-1.0d-1)
  call decompose(mydata,GS%onebody,GS%hgbr,GS%brbr,GS%hgbr)
  
  call Extract3body(myfunc,mydata)
  call convert_rkdata(myfunc,"rra")
  call find_alpha(myfunc,"dda")
  
  call date_and_time(sdate,stime1)
  call find_sigmas(GS%abs,myfunc) 
  call date_and_time(sdate,stime2)

  write(*,*) " Sigmas for abstraction piece   : ", get_sec(stime2)-get_sec(stime1)," sec."

end subroutine

! ==F====================================================F== ! 
Subroutine init_insert
  implicit none
  character (len = 30) :: sdate, stime1, stime2

  call read(mydata,"hgbr2-ins.dat")
  mydata%coordtype = "jac"
  call sort_fdata(mydata,-1.0d7,1.0d3,1.0d-1)
  call decompose(mydata,GS%onebody,GS%hgbr,GS%hgbr,GS%brbr)
  
  call Extract3body(myfunc,mydata)
  call convert_rkdata(myfunc,"crg")
  call find_alpha(myfunc,"tda")
  
  call date_and_time(sdate,stime1)
  call find_sigmas(GS%ins,myfunc)
  call date_and_time(sdate,stime2)
  write(*,*) " Sigmas for insertion piece     : ", get_sec(stime2)-get_sec(stime1)," sec."

end subroutine

! ==F====================================================F== !
Subroutine init_near_equil
  implicit none
  call read(GS%gmin,"hgbr2-eq.txt")
end subroutine

! ==F======================================================F== !
Subroutine init_finish
  implicit none
  character (len=30) :: filename = "hgbr2-pot.bin",sdate,stime
  
  call date_and_time(sdate,stime)
  GS%info = "Generated on "//sdate(5:6)//"-"//sdate(7:8)//"-"//sdate(1:4)//" at "//stime(1:2)//":"//stime(3:4)//":"//stime(5:6)
  call write_out(GS,filename)

  write(*,*) " HgBr2 potential written to file ", filename

end subroutine

end module

program main
   use surf_construct 
   implicit none

   write(*,*) " The HgBr2 PES initialization program."
   write(*,*) " Two-body terms."
   call set_default_param
   call init_twobody
   write(*,*) " Three-body parts. "
   call init_recomb
   call init_abstr
   call init_insert
   call init_near_equil
   call init_finish

end program


