
module hgbr2_surf_main

! ******************************************************************** !
! Here subs for calculations of HgBr2 surfaces                         !
!                                                                      !
!                                                                      !
!              r1 = r(HgBr), r2=r(HgBr), r3 = r(BrBr)                  ! 
!                                                                      !
! ******************************************************************** !                                                                      

  use switches
  use trigonom
  use coords
  use curve1d
  use surfitpol
  use fast_sum_def
  use fast_sum_eval
  use hgbr2_surf_def

  implicit none

contains

! *** three-body pieces and derivatives in rrr coordinates *** 


! ==F=================================================
Subroutine Rec_Pot_and_Der(x,y,z,ptn,vder)

  implicit none
  double precision, intent(in)                :: x,y,z
  double precision, intent(out)               :: ptn
  double precision, dimension(3), intent(out) :: vder

  double precision :: r1, r2, alp, en3body
  double precision,dimension(3) :: dr

  call rrr2rra(x,y,z,r1,r2,alp)

  call eval_fast(GS%rec,r1,r2,alp,en3body,dr)

  if (alp == 180.0d0) then

     ! removing 0/0 singularity in Jacobian of rra -> rrr

     call eval_fast_secder(GS%rec,r1,r2,alp,dr(3))
     vder(3) = -(dr(3)*z)/(x*y*dcos_der_der(alp))

  else
     vder(3) = -(dr(3)*z)/(x*y*dcos_der(alp))
  end if

  vder(1) = dr(1)-vder(3)*(x-y*dcos(alp))/z
  vder(2) = dr(2)-vder(3)*(y-x*dcos(alp))/z

  ptn =  en3body

end Subroutine


! ==F=================================================
Subroutine Abs_Pot_and_Der(x,y,z,ptn,vder)
  implicit none
  double precision, intent(in)                :: x,y,z
  double precision, intent(out)               :: ptn
  double precision, dimension(3), intent(out) :: vder

  double precision :: r1, r2, alp, en3body
  double precision,dimension(3) :: dr

  call rrr2rra(x,z,y,r1,r2,alp)

  call eval_fast(GS%abs,r1,r2,alp,en3body,dr)

  if (alp == 180.0d0) then

     ! removing 0/0 singularity in Jacobian of rra -> rrr

     call eval_fast_secder(GS%abs,r1,r2,alp,dr(3))
     vder(2) = -(dr(3)*y)/(x*z*dcos_der_der(alp))
  else
     vder(2) = -(dr(3)*y)/(x*z*dcos_der(alp))
  end if

  vder(1) = dr(1)-vder(2)*(x-z*dcos(alp))/y
  vder(3) = dr(2)-vder(2)*(z-x*dcos(alp))/y

  ptn = en3body 

end Subroutine


! ==F=================================================
Subroutine Ins_Pot_and_Der(x,y,z,ptn,vder)
  implicit none
  double precision, intent(in)                :: x,y,z
  double precision, intent(out)               :: ptn
  double precision, dimension(3), intent(out) :: vder

  double precision :: m,b,gam,d,c1,c2,c3, en3body
  double precision, dimension(3)   :: dr
  double precision, dimension(3,3) :: aa

  call rrr2jac(x,y,z,m,b,gam)

  call eval_fast(GS%ins,m,b,gam,en3body,dr)

  ! Jacobian of jac -> rrr transforation

  aa(1,1) =  (2.0*m-b*dcos(gam))/(2.0*x)
  aa(1,2) =  (2.0*m+b*dcos(gam))/(2.0*y)
  aa(1,3) =  0.0d0
  aa(2,1) =  (b/2.0-m*dcos(gam))/(2.0*x)
  aa(2,2) =  (b/2.0+m*dcos(gam))/(2.0*y)
  aa(2,3) =  1.0d0
  aa(3,1) = -m*b*dcos_der(gam)/(2.0*x)
  aa(3,2) =  m*b*dcos_der(gam)/(2.0*y)
  aa(3,3) =  0.0d0

  d  = aa(3,1)*aa(1,2)-aa(1,1)*aa(3,2)
  c1 = dr(3)*aa(1,2)-dr(1)*aa(3,2)
  c2 = aa(3,1)*dr(1)-aa(1,1)*dr(3)
  c3 =  aa(1,1)*aa(2,2)*dr(3)+aa(2,1)*aa(3,2)*dr(1)+aa(3,1)*aa(1,2)*dr(2) &
       -aa(3,1)*aa(2,2)*dr(1)-aa(2,1)*aa(1,2)*dr(3)-aa(1,1)*aa(3,2)*dr(2)

  vder(1) = c1/d
  vder(2) = c2/d
  vder(3) = c3/d

  ptn = en3body 

end Subroutine

! ==F================================================================F== !
Subroutine NearEq_Pot_and_Der(x,y,z,ptn,vder) 
   implicit none
   double precision, intent(in) :: x,y,z
   double precision, intent(out)               :: ptn
   double precision, dimension(3), intent(out) :: vder

   double precision, dimension(3) :: rr
   double precision :: r1, r2, alp
   double precision,dimension(3) :: dr

   call rrr2rra(x,y,z,r1,r2,alp)
   rr(1) = r1
   rr(2) = r2
   rr(3) = acos(-1.0d0)*alp/180.0d0

   ptn = fct(GS%gmin,rr)-GS%onebody

   call fct_der(GS%gmin,rr,dr)
   dr(3) = dr(3)*(acos(-1.0d0)/180.0d0)

    if (alp == 180.0d0) then
      ! removing 0/0 singularity in Jacobian of rra -> rrr
      call fct_secder(GS%gmin,rr,dr(3))
      dr(3)   =  dr(3)*(acos(-1.0d0)/180.0d0)**2.0
      vder(3) = -(dr(3)*z)/(x*y*dcos_der_der(alp))
    else
      vder(3) = -(dr(3)*z)/(x*y*dcos_der(alp))
    end if
  
    vder(1) = dr(1)-vder(3)*(x-y*dcos(alp))/z
    vder(2) = dr(2)-vder(3)*(y-x*dcos(alp))/z


end subroutine

! ==F================================================================F== !
Subroutine GlobalPotRKHS(r1in,r2in,r3in,ptn,gg)
!
!
! This one calcultes first connected 3-body piece
! cuts it to zero at long range 
! and adds 2-body terms
!
!
  implicit none
  double precision   :: r1in,r2in,r3in,ptn
  intent(in)         :: r1in,r2in,r3in
  intent(out)        :: ptn
  double precision, dimension(3), intent(out) :: gg


  double precision :: r1, r2, r3
  double precision :: alp1, alp2, alp3, gamma, h, beta , temp
  double precision :: erec, eins, eabs, erest, &
                             wrest, wrec, wins, &
                             wrec1, wins1, wabs, wcut

  double precision, dimension(3) ::   absder,    recder,   insder,  restder, &
                                    wabs_der, wrest_der, wins_der, wrec_der, &
                                   wins1_der, wrec1_der, wcut_der

  double precision, dimension(3) :: zero = (/0.0d0,0.0d0,0.0d0/)

  double precision :: thresh = 1.0d-7

  ! --- assuming r1 <= r2

  if (r1in <= r2in) then
     r1 = r1in
     r2 = r2in
     r3 = r3in
  else
     r1 = r2in
     r2 = r1in
     r3 = r3in
  end if

  alp3  = dacos((r1**2.0+r2**2.0-r3**2.0)/(2.0*r1*r2))   ! Br-Hg-Br angle
  beta  = dacos((r1**2.0+r3**2.0-r2**2.0)/(2.0*r1*r3))   ! it must be the largest Hg-Br-Br angle
  h     = mySqrt(r1**2.0+(r3/2.0)**2.0-r1*r3*dcos(beta)) ! median Hg-Br2
  gamma = dacos((h**2.0+(r3/2.0)**2.0-r1**2.0)/(h*r3))   ! angle Hg-midpoint-Br must be <= 90


  wabs  = 1.0d0-omega_switch(beta,GS%beta_sw,GS%beta_g)
  wrest = 1.0d0-wabs

  wrec1  = 1.0d0-omega_switch(alp3,GS%alp_sw,GS%alp_g)
  wins1  = 1.0d0 - wrec1

  wrec  = wrec1*wrest
  wins  = wins1*wrest

  ! --- calculating 3-body terms for different regions

  if (abs(wabs) > thresh) then
     call Abs_Pot_and_Der(r1,r2,r3,eabs,absder)
  else
     eabs   = 0.0d0
     absder = zero
  end if

  if (abs(wins) > thresh) then
      call Ins_Pot_and_Der(r1,r2,r3,eins,insder)
  else
      eins   = 0.0d0
      insder = zero
  end if

  if (abs(wrec) > thresh) then
     call Rec_Pot_and_Der(r1,r2,r3,erec,recder)
  else
     erec   = 0.0d0
     recder = zero
  end if

  ptn = wabs*eabs + wins*eins + wrec*erec 

  ! --- derivatives of switches along (r1,r2,r3) coordinates

  if (abs(wabs) > thresh) then
      if (beta == 180.0) then
         wabs_der(2) = omega_switch_der_der(beta,GS%beta_sw,GS%beta_g)*r2/(r1*r3*dcos_der_der(beta))
      else
         wabs_der(2) = omega_switch_der(beta,GS%beta_sw,GS%beta_g)*r2/(r1*r3*dcos_der(beta))
      end if 
      wabs_der(1) = -wabs_der(2)*(r1-r3*dcos(beta))/r2
      wabs_der(3) = -wabs_der(2)*(r3-r1*dcos(beta))/r2
  else
       wabs_der = zero
  end if
  wrest_der = -1.0d0*wabs_der

  if (abs(wrec1) > thresh) then
      if (alp3 == 180.0) then
         wrec1_der(3) = omega_switch_der_der(alp3,GS%alp_sw,GS%alp_g)*r3/(r1*r2*dcos_der_der(alp3))
      else
         wrec1_der(3) = omega_switch_der(alp3,GS%alp_sw,GS%alp_g)*r3/(r1*r2*dcos_der(alp3))
      end if  
      wrec1_der(1) = -wrec1_der(3)*(r1-r2*dcos(alp3))/r3
      wrec1_der(2) = -wrec1_der(3)*(r2-r1*dcos(alp3))/r3
  else
      wrec1_der = zero
  end if
  wins1_der = -1.0d0*wrec1_der

  wrec_der = wrest*wrec1_der + wrec1*wrest_der
  wins_der = wrest*wins1_der + wins1*wrest_der

  gg =       wabs*absder + eabs*wabs_der &
          +  wins*insder + eins*wins_der &
          +  wrec*recder + erec*wrec_der

  if (r1in > r2in) then
      temp = gg(1); gg(1) = gg(2); gg(2) = temp
  end if


  ! --- cutting long range 3-body potental to zero

  wcut =   omega_switch(r1in,GS%r1max,GS%g1max)  &
         * omega_switch(r2in,GS%r2max,GS%g2max)  &
         * omega_switch(r3in,GS%r3max,GS%g3max)
  
  wcut_der(1) =    omega_switch_der(r1in,GS%r1max,GS%g1max)  &
                 * omega_switch(r2in,GS%r2max,GS%g2max)      &
                 * omega_switch(r3in,GS%r3max,GS%g3max)

  wcut_der(2) =    omega_switch(r1in,GS%r1max,GS%g1max)      &
                 * omega_switch_der(r2in,GS%r2max,GS%g2max)  &
                 * omega_switch(r3in,GS%r3max,GS%g3max)

  wcut_der(1) =    omega_switch(r1in,GS%r1max,GS%g1max)  &
                 * omega_switch(r2in,GS%r2max,GS%g2max)  &
                 * omega_switch_der(r3in,GS%r3max,GS%g3max)


  ptn = wcut*ptn

  gg = ptn*wcut_der + wcut*gg

  
  ! --- adding 2-body terms

  ptn = ptn + rkhs1d_value(GS%HgBr,r1in) &
            + rkhs1d_value(GS%HgBr,r2in) &
            + rkhs1d_value(GS%BrBr,r3in)

  gg(1) = gg(1) + rkhs1d_value_der(GS%HgBr,r1in) 
  gg(2) = gg(2) + rkhs1d_value_der(GS%HgBr,r2in)
  gg(3) = gg(3) + rkhs1d_value_der(GS%BrBr,r3in)


end subroutine


! ==F================================================================F== !
Subroutine GlobalPot(r1,r2,r3,ptn,gg)
!
! Here we put patch on near HgBr2 equilibrium geometry
!
  implicit none
  double precision, intent(in)                          ::  r1,r2,r3
  double precision, intent(out)                         ::  ptn
  double precision, dimension(3), intent(out), optional :: gg
  
  double precision                ::  weq, eneq, wrest, enrest, alp, ro, omega_der
  double precision, dimension(3)  ::  weq_rrr, weq_rra, wrest_rrr, eneq_rrr, enrest_rrr
  double precision, dimension(3)  ::  zero = (/0.0d0,0.0d0,0.0d0/)
  double precision, parameter     ::  thresh = 1.0d-8
  
  alp = dacos((r1**2.0+r2**2.0-r3**2.0)/(2.0*r1*r2))

  ! there is a bug when alp == 180.0d0

  if (alp == 180.0d0) then
    alp = 179.999999d0
  end if
  
  ro  =   mySqrt(   ((r1-GS%r1eq)/GS%dlt1eq)**2.0      &
                  + ((r2-GS%r2eq)/GS%dlt2eq)**2.0      &
                  + ((alp-GS%alp_eq)/GS%dlta_eq)**2.0  &
                  + GS%singcorr                        &
                )
  
  weq   = omega_switch(ro,GS%ro_eq,GS%gamma_eq)
  wrest = 1.0d0-weq

  if (abs(weq) > thresh) then
     call NearEq_Pot_and_Der(r1,r2,r3,eneq,eneq_rrr)
  else
     eneq     = 0.0d0
     eneq_rrr = zero
  end if

  if (abs(wrest) > thresh) then
    call GlobalPotRKHS(r1,r2,r3,enrest,enrest_rrr)
  else
    enrest = 0.0d0
    enrest_rrr = zero
  end if

  omega_der  = omega_switch_der(ro,GS%ro_eq,GS%gamma_eq)

  weq_rra(1) = omega_der * (r1-GS%r1eq)    / (ro*GS%dlt1eq**2.0)
  weq_rra(2) = omega_der * (r2-GS%r2eq)    / (ro*GS%dlt2eq**2.0)
  weq_rra(3) = omega_der * (alp-GS%alp_eq) / (ro*GS%dlta_eq**2.0)

  if ((alp == 180.0d0).or.(alp == 0.0d0)) then
     weq_rra(3) =   omega_switch_der_der(ro,GS%ro_eq,GS%gamma_eq)*(1.0d0/(ro*GS%dlta_eq**2.0))
     weq_rrr(3) = -(weq_rra(3)*r3)/(r1*r2*dcos_der_der(alp))
  else
     weq_rrr(3) = -(weq_rra(3)*r3)/(r1*r2*dcos_der(alp))
  end if
  
  weq_rrr(1) =  weq_rra(1)-weq_rrr(3)*(r1-r2*dcos(alp))/r3
  weq_rrr(2) =  weq_rra(2)-weq_rrr(3)*(r2-r1*dcos(alp))/r3

  wrest_rrr  =  - weq_rrr

  ptn = weq * eneq + wrest * enrest 
  gg  = eneq*weq_rrr + weq*eneq_rrr + enrest*wrest_rrr + wrest*enrest_rrr

!  ptn = eneq
!  gg  = eneq_rrr 


end Subroutine

! ==F================================================================F== !
Subroutine GlobalPotCart(aa,bb,cc,ptn,dx)
!
! This one will be called from HookUp's to POTLIB20001
!
  implicit none
  double precision, dimension(3), intent(in)  :: aa,bb,cc
  double precision, dimension(9), intent(out) :: dx
  double precision, intent(out)               :: ptn

  double precision, dimension(3)   :: rr,gg
  double precision, dimension(9,3) :: jcb

  call Cart2rrr(aa,bb,cc,rr,jcb)
  call GlobalPot(rr(1),rr(2),rr(3),ptn,gg)
  dx = matmul(jcb,gg)

end Subroutine

! ==F================================================================F== !
Subroutine GlobalPotCartDer(xx,dx)
!
! Numerical derivatives using 5-points central formula,
! (just for testing)
!
  implicit none
  double precision, dimension(9), intent(in) :: xx
  double precision, dimension(9), intent(out), optional :: dx

  double precision :: h, v1,v2,v3,v4
  double precision, dimension(9) :: yy
  double precision, dimension(9) :: vder
  integer :: i

  h=0.0005d0
  do i=1,9

    yy=xx
    yy(i)=yy(i)-2.0*h
    call GlobalPotCart(yy(1:3),yy(4:6),yy(7:9),v1,vder)

    yy=xx
    yy(i)=yy(i)-h
    call GlobalPotCart(yy(1:3),yy(4:6),yy(7:9),v2,vder)

    yy=xx
    yy(i)=yy(i)+h
    call GlobalPotCart(yy(1:3),yy(4:6),yy(7:9),v3,vder)

    yy=xx
    yy(i)=yy(i)+2.0*h
    call GlobalPotCart(yy(1:3),yy(4:6),yy(7:9),v4,vder)

    dx(i) = (v1-8.0*v2+8.0*v3-v4)/(12.0*h)

  end do

end Subroutine


end module

